@@ -12,36 +12,11 @@ import os
 import sys
 import typing as tp
 from collections import defaultdict
+from typing import Annotated
 
 from xonsh.built_ins import XSH
 from xonsh.completers.tools import RichCompletion
 
-TYPING_ANNOTATED_AVAILABLE = False
-"""One can import ``Annotated`` from this module
-which adds a stub when it is not available in ``typing``/``typing_extensions`` modules."""
-
-try:
-    from typing import Annotated  # noqa
-
-    TYPING_ANNOTATED_AVAILABLE = True
-except ImportError:
-    try:
-        from typing_extensions import Annotated  # type: ignore
-
-        TYPING_ANNOTATED_AVAILABLE = True
-    except ImportError:
-        T = tp.TypeVar("T")  # Declare type variable
-
-        class _AnnotatedMeta(type):
-            def __getitem__(self, item: tp.Tuple[T, tp.Any]) -> T:
-                if tp.TYPE_CHECKING:
-                    return item[0]
-
-                return item[1]
-
-        class Annotated(metaclass=_AnnotatedMeta):  # type: ignore
-            pass
-
 
 class ArgCompleter:
     """Gives a structure to the argparse completers"""
@@ -114,8 +89,8 @@ class NumpyDoc:
         return NumpyDoc.join(desc), lines[idx + 2 :]
 
     @staticmethod
-    def get_param_doc(lines: tp.List[str]):
-        docs: tp.Dict[str, tp.List[str]] = defaultdict(list)
+    def get_param_doc(lines: list[str]):
+        docs: dict[str, list[str]] = defaultdict(list)
         name = None
 
         while lines:
@@ -139,15 +114,15 @@ class NumpyDoc:
 _FUNC_NAME = "_func_"
 
 
-def _get_args_kwargs(annot: tp.Any) -> tp.Tuple[tp.Sequence[str], tp.Dict[str, tp.Any]]:
+def _get_args_kwargs(annot: tp.Any) -> tuple[tp.Sequence[str], dict[str, tp.Any]]:
     args, kwargs = [], {}
     if isinstance(annot, tuple):
         args, kwargs = annot
-    elif TYPING_ANNOTATED_AVAILABLE and "Annotated[" in str(annot):
+    elif "Annotated[" in str(annot):
         if hasattr(annot, "__metadata__"):
             args, kwargs = annot.__metadata__[0]
         else:
-            from typing_extensions import get_args
+            from typing import get_args
 
             _, (args, kwargs) = get_args(annot)
 
@@ -202,7 +177,7 @@ def add_args(
             # for booleans set action automatically
             if (
                 flags
-                and (type(param.default) == bool)
+                and isinstance(param.default, bool)
                 and ("action" not in kwargs)
                 and ("type" not in kwargs)
             ):
@@ -214,23 +189,11 @@ def add_args(
         kwargs.setdefault("help", doc.params.get(name))
 
         completer = kwargs.pop("completer", None)
-        cls_inst = getattr(func, "__self__", None)
-
-        if cls_inst and isinstance(cls_inst, ArgParserAlias):
-            flags, kwargs = cls_inst.hook_pre_add_argument(
-                param=name, func=func, flags=flags, kwargs=kwargs
-            )
-
         action = parser.add_argument(*flags, **kwargs)
 
         if completer:
             action.completer = completer  # type: ignore
 
-        if cls_inst and isinstance(cls_inst, ArgParserAlias):
-            cls_inst.hook_post_add_argument(
-                parser=parser, action=action, param=name, func=func
-            )
-
         action.help = action.help or ""
         # Don't show default when
         # 1. None : No value is given for the option
@@ -245,9 +208,13 @@ def add_args(
             action.help += " (type: %(type)s)"
 
 
+def empty_help_func(_parser, _stdout):
+    _parser.print_help(file=_stdout)
+
+
 def make_parser(
     func: tp.Union[tp.Callable, str],
-    empty_help=True,
+    empty_help=False,
     **kwargs,
 ) -> "ArgParser":
     """A bare-bones argparse builder from functions"""
@@ -259,9 +226,7 @@ def make_parser(
             kwargs["epilog"] = doc.epilog
     parser = ArgParser(**kwargs)
     if empty_help:
-        parser.set_defaults(
-            **{_FUNC_NAME: lambda stdout=None: parser.print_help(file=stdout)}
-        )
+        parser.set_defaults(**{_FUNC_NAME: empty_help_func})
     return parser
 
 
@@ -342,9 +307,14 @@ class ArgParser(ap.ArgumentParser):
 
         super().__init__(**kwargs)
         self.commands = None
+        self.default_command = None
 
     def add_command(
-        self, func: tp.Callable, args: tp.Optional[tp.Iterable[str]] = None, **kwargs
+        self,
+        func: tp.Callable,
+        default=False,
+        args: "tuple[str, ...] | None" = None,
+        **kwargs,
     ):
         """
             create a sub-parser and call this function during dispatch
@@ -358,6 +328,8 @@ class ArgParser(ap.ArgumentParser):
             Use _parser to get the generated parser instance.
             Use _args to get what is passed from sys.argv
             Use _parsed to get result of ``parser.parse_args``
+        default
+            Marks this sub-command as the default command for this parser.
         args
             if given only add these arguments to the parser.
             Otherwise all parameters to the function without `_` prefixed
@@ -378,12 +350,27 @@ class ArgParser(ap.ArgumentParser):
         name = kwargs.pop("prog", None)
         if not name:
             name = func.__name__.lstrip("_").replace("_", "-")
+
+        if default:
+            self.default_command = name
+
         parser = self.commands.add_parser(name, **kwargs)
-        add_args(parser, func, allowed_params=args)
+        add_args(parser, func, allowed_params=args, doc=doc)
         return parser
 
+    def _parse_known_args(self, arg_strings: list[str], namespace: ap.Namespace):
+        arg_set = set(arg_strings)
+        if (
+            self.commands
+            and self.default_command
+            and ({"-h", "--help"}.isdisjoint(arg_set))
+            and (set(self.commands.choices).isdisjoint(arg_set))
+        ):
+            arg_strings = [self.default_command] + arg_strings
+        return super()._parse_known_args(arg_strings, namespace)
 
-def _dispatch_func(func: tp.Callable, ns: tp.Dict[str, tp.Any]):
+
+def _dispatch_func(func: tp.Callable, ns: dict[str, tp.Any]):
     """Final dispatch to the function based on signature."""
     sign = inspect.signature(func)
     kwargs = {}
@@ -412,6 +399,8 @@ def dispatch(parser: ap.ArgumentParser, args=None, lenient=False, **ns):
     ns
         a dict that will be passed to underlying function
     """
+    ns.setdefault("_parser", parser)
+    ns.setdefault("_args", args)
 
     if lenient:
         parsed, unparsed = parser.parse_known_args(args)
@@ -440,7 +429,7 @@ class ArgparseCompleter:
         self.kwargs = kwargs
 
     @staticmethod
-    def get_parser(parser, args) -> tp.Tuple[ap.ArgumentParser, tp.Tuple[str, ...]]:
+    def get_parser(parser, args) -> tuple[ap.ArgumentParser, tuple[str, ...]]:
         """Check for sub-parsers"""
         sub_parsers = {}
         for act in parser._get_positional_actions():
@@ -600,37 +589,21 @@ class ArgParserAlias:
         self.stdout = None
         self.stderr = None
 
-    def build(self):
+    def build(self) -> "ArgParser":
         """Sub-classes should return constructed ArgumentParser"""
         if self.kwargs:
             return self.create_parser(**self.kwargs)
         raise NotImplementedError
 
-    def hook_pre_add_argument(self, param: str, func, flags, kwargs):
-        """Hook to update arguments that are passed to parser.add_argument"""
-        return flags, kwargs
-
-    def hook_post_add_argument(
-        self,
-        *,
-        parser: "ArgParser|ap.ArgumentParser",
-        action: "ap.Action",
-        param: str,
-        func: "tp.Callable",
-    ):
-        """Hook into parser.add_argument step.
-
-        Can be used to update action's attributes
-        """
-        return
-
     @property
     def parser(self):
         if self._parser is None:
             self._parser = self.build()
         return self._parser
 
-    def create_parser(self, func=None, has_args=False, allowed_params=None, **kwargs):
+    def create_parser(
+        self, func=None, has_args=False, allowed_params=None, **kwargs
+    ) -> "ArgParser":
         """create root parser"""
         func = func or self
         has_args = has_args or bool(allowed_params)
@@ -676,8 +649,6 @@ class ArgParserAlias:
             result = dispatch(
                 self.parser,
                 args,
-                _parser=self.parser,
-                _args=args,
                 _stdin=stdin,
                 _stdout=stdout,
                 _stderr=stderr,
