@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Key bindings for prompt_toolkit xonsh shell."""
 
 from prompt_toolkit import search
@@ -6,24 +5,26 @@ from prompt_toolkit.application.current import get_app
 from prompt_toolkit.enums import DEFAULT_BUFFER
 from prompt_toolkit.filters import (
     Condition,
-    IsMultiline,
-    HasSelection,
     EmacsInsertMode,
-    ViInsertMode,
+    HasSelection,
+    IsMultiline,
     IsSearching,
+    ViInsertMode,
 )
-from prompt_toolkit.keys import Keys
-from prompt_toolkit.key_binding.key_bindings import KeyBindings, KeyBindingsBase
+from prompt_toolkit.input import ansi_escape_sequences
 from prompt_toolkit.key_binding.bindings.named_commands import get_by_name
+from prompt_toolkit.key_binding.key_bindings import KeyBindings, KeyBindingsBase
+from prompt_toolkit.keys import Keys
 
 from xonsh.aliases import xonsh_exit
+from xonsh.built_ins import XSH
+from xonsh.platform import ON_WINDOWS
+from xonsh.shell import transform_command
 from xonsh.tools import (
     check_for_partial_string,
-    get_line_continuation,
     ends_with_colon_token,
+    get_line_continuation,
 )
-from xonsh.built_ins import XSH
-from xonsh.shell import transform_command
 
 DEDENT_TOKENS = frozenset(["raise", "return", "pass", "break", "continue"])
 
@@ -208,15 +209,46 @@ def wrap_selection(buffer, left, right=None):
     buffer.selection_state = selection_state
 
 
-def load_xonsh_bindings() -> KeyBindingsBase:
+def load_xonsh_bindings(ptk_bindings: KeyBindingsBase) -> KeyBindingsBase:
     """
     Load custom key bindings.
+
+    Parameters
+    ----------
+    ptk_bindings :
+        The default prompt toolkit bindings. We need these to add aliases to them.
     """
     key_bindings = KeyBindings()
     handle = key_bindings.add
     has_selection = HasSelection()
     insert_mode = ViInsertMode() | EmacsInsertMode()
 
+    if XSH.env["XONSH_CTRL_BKSP_DELETION"]:
+        # Not all terminal emulators emit the same keys for backspace, therefore
+        # ptk always maps backspace ("\x7f") to ^H ("\x08"), and all the backspace bindings are registered for ^H.
+        # This means we can't re-map backspace and instead we register a new "real-ctrl-bksp" key.
+        # See https://github.com/xonsh/xonsh/issues/4407
+
+        if ON_WINDOWS:
+            # On windows BKSP is "\x08" and CTRL-BKSP is "\x7f"
+            REAL_CTRL_BKSP = "\x7f"
+
+            # PTK uses a second mapping
+            from prompt_toolkit.input import win32 as ptk_win32
+
+            ptk_win32.ConsoleInputReader.mappings[b"\x7f"] = REAL_CTRL_BKSP  # type: ignore
+        else:
+            REAL_CTRL_BKSP = "\x08"
+
+        # Prompt-toolkit allows using single-character keys that aren't in the `Keys` enum.
+        ansi_escape_sequences.ANSI_SEQUENCES[REAL_CTRL_BKSP] = REAL_CTRL_BKSP  # type: ignore
+        ansi_escape_sequences.REVERSE_ANSI_SEQUENCES[REAL_CTRL_BKSP] = REAL_CTRL_BKSP  # type: ignore
+
+        @handle(REAL_CTRL_BKSP, filter=insert_mode)
+        def delete_word(event):
+            """Delete a single word (like ALT-backspace)"""
+            get_by_name("backward-kill-word").call(event)
+
     @handle(Keys.Tab, filter=tab_insert_indent)
     def insert_indent(event):
         """
@@ -389,4 +421,16 @@ def load_xonsh_bindings() -> KeyBindingsBase:
             buff.cut_selection()
         get_by_name("yank").call(event)
 
+    def create_alias(new_keys, original_keys):
+        bindings = ptk_bindings.get_bindings_for_keys(tuple(original_keys))
+        for original_binding in bindings:
+            handle(*new_keys, filter=original_binding.filter)(original_binding.handler)
+
+    # Complete a single auto-suggestion word
+    create_alias([Keys.ControlRight], ["escape", "f"])
+
+    # since macOS uses Control as reserved, then we use the alt/option key instead
+    # which is mapped as the "escape" key
+    create_alias(["escape", "right"], ["escape", "f"])
+
     return key_bindings
