@@ -16,6 +16,7 @@ Implementations:
 * indent()
 
 """
+
 import ast
 import collections
 import collections.abc as cabc
@@ -227,8 +228,24 @@ class EnvPath(cabc.MutableSequence):
     def __delitem__(self, key):
         self._l.__delitem__(key)
 
+    @staticmethod
+    def _prepare_path(p):
+        return str(expand_path(p))
+
     def insert(self, index, value):
-        self._l.insert(index, value)
+        self._l.insert(index, self._prepare_path(value))
+
+    def append(self, value):
+        self._l.append(self._prepare_path(value))
+
+    def prepend(self, value):
+        self._l.insert(0, self._prepare_path(value))
+
+    def remove(self, value):
+        try:
+            self._l.remove(self._prepare_path(value))
+        except ValueError:
+            print(f"EnvPath warning: path {repr(value)} not found.", file=sys.stderr)
 
     @property
     def paths(self):
@@ -291,7 +308,7 @@ class EnvPath(cabc.MutableSequence):
         None
 
         """
-        data = str(expand_path(data))
+        data = self._prepare_path(data)
         if data not in self._l:
             self._l.insert(0 if front else len(self._l), data)
         elif replace:
@@ -1108,7 +1125,8 @@ def display_colored_error_message(exc_info, strip_xonsh_error_types=True, limit=
     # color the traceback if available
     _, interactive = _get_manual_env_var("XONSH_INTERACTIVE", 0)
     _, color_results = _get_manual_env_var("COLOR_RESULTS", 0)
-    if not interactive or not color_results or not HAS_PYGMENTS:
+    shell_loaded = hasattr(xsh.shell, "shell")  # do not color traceback in xonshrc file
+    if not interactive or not color_results or not shell_loaded or not HAS_PYGMENTS:
         sys.stderr.write(traceback_str)
         return
 
@@ -2708,7 +2726,7 @@ def columnize(elems, width=80, newline="\n"):
     data = [elems[i * nrows : (i + 1) * nrows] for i in range(ncols)]
     colwidths = [max(map(len, d)) + pad for d in data]
     colwidths[-1] -= pad
-    row_t = "".join(["{{row[{i}]: <{{w[{i}]}}}}".format(i=i) for i in range(ncols)])
+    row_t = "".join([f"{{row[{i}]: <{{w[{i}]}}}}" for i in range(ncols)])
     row_t += newline
     lines = [
         row_t.format(row=row, w=colwidths)
